@@ -8,6 +8,7 @@ repository: http://github.com/TiddlySpace/tiddlyspace
 
 
 import tiddlywebplugins.status
+import tiddlyweb.web.util
 
 from tiddlyweb.model.user import User
 from tiddlyweb.store import NoUserError
@@ -19,10 +20,11 @@ from tiddlywebplugins.instancer.util import get_tiddler_locations
 from tiddlywebplugins.tiddlyspace.commands import establish_commands
 from tiddlywebplugins.tiddlyspace.config import config as space_config
 from tiddlywebplugins.tiddlyspace.instance import store_contents
+from tiddlywebplugins.tiddlyspace.space import Space
 from tiddlywebplugins.tiddlyspace.www import establish_www
 
 
-__version__ = '1.0.37'
+__version__ = '1.0.38'
 
 
 def init(config):
@@ -126,3 +128,44 @@ def _status_gather_data(environ):
 
 
 tiddlywebplugins.status._gather_data = _status_gather_data
+
+
+original_tiddler_url = tiddlyweb.web.util.tiddler_url
+
+
+def web_tiddler_url(environ, tiddler, container='bags', full=True):
+    """
+    Override default tiddler_url to be space+host aware.
+
+    If the bag or recipe of the tiddler is of a space, switch to
+    that space's host for the duration of uri creation.
+
+    Do this all the time, so that we get the right URIs even
+    when working around ControlView.
+    """
+    saved_host = environ.get('HTTP_HOST', '')
+    try:
+        if container == 'recipes':
+            space_name = Space.name_from_recipe(tiddler.recipe)
+        else:
+            space_name = Space.name_from_bag(tiddler.bag)
+
+        host = environ['tiddlyweb.config']['server_host']['host']
+        port = environ['tiddlyweb.config']['server_host']['port']
+        if port is '443' or port is '80':
+            port = ''
+        else:
+            port = ':%s' % port
+        environ['HTTP_HOST'] = '%s.%s%s' % (space_name.encode('utf-8'),
+            host, port)
+    except ValueError:
+        pass
+    url = original_tiddler_url(environ, tiddler, container, full)
+    if saved_host:
+        environ['HTTP_HOST'] = saved_host
+    elif 'HTTP_HOST' in environ:
+        del environ['HTTP_HOST']
+    return url
+
+
+tiddlyweb.web.util.tiddler_url = web_tiddler_url
